@@ -1,5 +1,5 @@
 """Top-level package for Wailord."""
 
 __author__ = """Rohit Goswami"""
-__email__ = 'rog32@hi.is'
-__version__ = '0.0.1'
+__email__ = "rog32@hi.is"
+__version__ = "0.0.2"
